==> schedule-0.1.3.jar: schedule.clj <==
(ns schedule
        (:require [clojure.string :as str])
        (:import clojure.lang.IDeref
                 clojure.lang.Seqable
                 clojure.lang.Cons
                 clojure.lang.ILookup
                 java.util.Date
                 java.util.Calendar
                 java.util.TimeZone
                 java.io.Writer)
                                           
                                            )

(defprotocol Pattern
  (anchor  [pattern t]    "Anchors the scheduling pattern at time `t`, yielding a schedule starting after `t`"))

(defprotocol WeeklyTriggered
  (at-hour [pattern hour] "Return a new pattern that only generates instants at hour `hour`")
  (add-hour [pattern hour] "Returns a new pattern that generates instants at hour `hour`")
  (remove-hour [pattern hour] "Returns a new pattern that will not generate instants at hour `hour`")
  (at-time [pattern hour minute] "Return a new pattern that only generates instants at the time indicated by `hour` and `minute`")
  (add-time [pattern hour minute] "Returns a new pattern that generates instants at the time indicated by `hour` and `minute")
  (remove-time [pattern hour minute] "Returns a new pattern that will not generate instants at the time indicated by `hour` and `minute`"))

(defprotocol TimeZoneable
  (in-tz [timezonable tz] "Shifts timezoneable, returning a new object projected in timezoe `tz`."))

(defprotocol Instant
  (as-ts [instant] "Return a plaform native timestamp type corresponding to this instant.")
  (as-millis [instant] "Return a long representing the number of millis since the epoch corresponding to this instant."))

(defn current-time-millis
  []
        (System/currentTimeMillis)
                       
  )

      (extend-protocol Instant
        Date
        (as-ts [date] date)
        (as-millis [date] (.getTime date))
        Long
        (as-ts [long] (Date. long))
        (as-millis [long] long)
        Calendar
        (as-ts [cal] (.getTime cal))
        (as-millis [cal] (.getTimeInMillis cal)))

                               
                
                            
                                           
               
                                     
                               

(declare ^:private update-schedule-pattern)
(declare schedule-lazy-seq)

                      
                 
                 
                 
                 
                 
                 
                 
                   

(defn roll-to
  "Rolls ms forward, finding the next time ms which occurs on day at
  hour and minute in tz"
  [ms day hour minute tz]
        (let [weekday-map {:monday Calendar/MONDAY
                           :tuesday Calendar/TUESDAY
                           :wednesday Calendar/WEDNESDAY
                           :thursday Calendar/THURSDAY
                           :friday Calendar/FRIDAY
                           :saturday Calendar/SATURDAY
                           :sunday Calendar/SUNDAY}
              cal (Calendar/getInstance)]
          (.setTimeInMillis cal ms)
          (.set cal Calendar/DAY_OF_WEEK (weekday-map day))
          (.set cal Calendar/HOUR_OF_DAY hour)
          (.set cal Calendar/MINUTE minute)
          (.set cal Calendar/SECOND 0)
          (.set cal Calendar/MILLISECOND 0)
          (.setTimeZone cal (TimeZone/getTimeZone (or tz "UTC")))
          (if (<= ms (.getTimeInMillis cal))
            (.getTimeInMillis cal)
            (do (.add cal Calendar/DAY_OF_YEAR 7)
                (.getTimeInMillis cal))))
                                     
                                      
                                        
                                       
                                     
                                       
                                      
                              
                                               
                                                              
                                                           
                                                          
                                                                   
                                                       
                                              
                                         
                                           
                                                             
                                                               
                                           
                                 
                                                              
                                        )

(defn- calculate-next-schedule-instant
  [{{:keys [tz time-matches days] :as pattern} :pattern
    start :start
    :as schedule}]
  (let [days (or days
                 [:monday :tuesday :wednesday :thursday :friday :saturday :sunday])
        candidates (for [day days [hour min] time-matches]
                     (roll-to start day hour min tz))]
    (apply min candidates)))

(defn print-to-writer
  [val writer opts]
                                     
        (binding [*out* writer]
          (pr val)))

(defn print-weekly-schedule-to-abstract-writer
  [{:keys [pattern start] :as schedule} writer write-fn opts]
  (write-fn writer "#schedule/weekly-schedule ")
  (print-to-writer {:pattern pattern
                    :start (as-ts start)} writer opts))

(defn- lookup-schedule-key
  ([schedule key] (lookup-schedule-key schedule key nil))
  ([schedule key not-found]
     (let [lookup (condp get key
                    #{:pattern 'pattern} (.-pattern schedule)
                    #{:start 'start} (.-start schedule)
                    ::not-found)]
       (if (= lookup ::not-found)
         not-found
         lookup))))

(deftype WeeklySchedule [pattern start]
  Pattern
  (anchor  [this t]    (WeeklySchedule. pattern (as-millis t)))
  WeeklyTriggered
  (at-hour [this hour] (update-schedule-pattern at-hour hour))
  (add-hour [this hour] (update-schedule-pattern add-hour hour))
  (remove-hour [this hour] (update-schedule-pattern remove-hour hour))
  (at-time [this hour minute] (update-schedule-pattern at-time hour minute))
  (add-time [this hour minute] (update-schedule-pattern add-time hour minute))
  (remove-time [this hour minute] (update-schedule-pattern remove-time hour minute))
  TimeZoneable
  (in-tz   [this tz]   (update-schedule-pattern in-tz tz))
        ILookup
        (valAt [this key] (lookup-schedule-key this key))
        (valAt [this key not-found] (lookup-schedule-key this key not-found))
                
                                                            
                                                                                
        Seqable
        (seq [this] (schedule-lazy-seq this))
                 
                                               
                         
                                           
                                                                                  )

      (defmethod print-method WeeklySchedule
        [^WeeklySchedule schedule ^Writer w]
        (print-weekly-schedule-to-abstract-writer schedule w #(.write %1 %2) nil))

(defn schedule-lazy-seq
  [{:keys [pattern] :as schedule}]
  (let [next-instant-ms (calculate-next-schedule-instant schedule)]
    (cons (as-ts next-instant-ms)
          (lazy-seq (WeeklySchedule. pattern (inc next-instant-ms))))))

(defn- update-schedule-pattern
  [{:keys [pattern start]} f & args]
  (WeeklySchedule. (apply f pattern args) start))

(defn- write-n
  [coll writer write-fn serial-fn empty-fn]
  (case (count coll)
    0 (write-fn writer (empty-fn))
    1 (write-fn writer (serial-fn (first coll)))
    2 (do (write-fn writer (serial-fn (first coll)))
          (write-fn writer ", and ")
          (write-fn writer (serial-fn (second coll))))
    (do (doseq [item (butlast coll)]
          (write-fn writer (serial-fn item))
          (write-fn writer ", "))
        (write-fn writer "and ")
        (write-fn writer (serial-fn (last coll))))))

(defn- print-weekly-pattern-to-abstract-writer
  [{:keys [days time-matches tz] :as pattern} writer write-fn opts]
  (write-fn writer "#schedule/weekly-pattern \"")
  (write-n days writer write-fn #(str/capitalize (name %)) (constantly "Every day"))
  (when time-matches
    (write-fn writer " at "))
  (write-n (sort time-matches) writer write-fn #(let [[hour minute] %]
                                                  (str hour ":" (when (< minute 10) "0") minute))
           (constantly ""))
  (when tz
    (write-fn writer " ")
    (write-fn writer tz))
  (write-fn writer "\""))

(defn- lookup-pattern-key
  ([pattern key] (lookup-pattern-key pattern key nil))
  ([pattern key not-found]
     (let [lookup (condp get key
                    #{:days 'days} (.-days pattern)
                    #{:time-matches 'time-matches} (.-time-matches pattern)
                    #{:tz 'tz} (.-tz pattern)
                    ::not-found)]
       (if (= lookup ::not-found)
         not-found
         lookup))))

(deftype WeeklyPattern [days time-matches tz]
  Pattern
  (anchor  [pattern t]    (WeeklySchedule. pattern (as-millis t)))
  WeeklyTriggered
  (at-hour [pattern new-hour] (WeeklyPattern. days #{[new-hour 0]} tz))
  (add-hour [pattern new-hour] (WeeklyPattern. days (conj time-matches [new-hour 0]) tz))
  (remove-hour [pattern del-hour] (WeeklyPattern. days (disj time-matches [del-hour 0]) tz))
  (at-time [pattern new-hour new-minute] (WeeklyPattern. days #{[new-hour new-minute]} tz))
  (add-time [pattern new-hour new-minute] (WeeklyPattern. days (conj time-matches [new-hour new-minute]) tz))
  (remove-time [pattern del-hour del-minute] (WeeklyPattern. days (disj time-matches [del-hour del-minute]) tz))
  TimeZoneable
  (in-tz   [pattern new-tz]   (WeeklyPattern. days time-matches new-tz))
        ILookup
        (valAt [this key] (lookup-pattern-key this key))
        (valAt [this key not-found] (lookup-pattern-key this key not-found))
                
                                                           
                                                                               
        IDeref
        (deref [pattern] (anchor pattern (current-time-millis)))
               
                                                                  
                         
                                          
                                                                                )

      (defmethod print-method WeeklyPattern
        [^WeeklyPattern pattern ^Writer w]
        (print-weekly-pattern-to-abstract-writer pattern w #(.write %1 %2) nil))

(defn read-schedule
  [m]
  (let [pattern (:pattern m)
        start (:start m)]
    (WeeklySchedule. pattern (as-millis start))))

(defn- string->int
  [int]
  (when int
          (Integer/parseInt int)
                               ))

(defn- floating-pattern
  [days time-matches tz]
  (WeeklyPattern. days time-matches tz))

(defn- match-globs
  [re s]
        (let [matcher (re-matcher re s)]
          (re-find matcher))
                          )

(defn pull-days
  ([s]
     (pull-days s nil))
  ([s days]
     (let [day-re #"((?:, )?(?:and )?((?:Every |Mon|Tues|Wednes|Thurs|Fri|Satur|Sun)day)).*"
           [match root first-day] (match-globs day-re s)]
       (if (or (= first-day "Every day")
               (nil? first-day))
         {:days days
          :days-rest (.substring s (count root))}
         (recur (.substring s (count root))
                (conj days (keyword (.toLowerCase first-day))))))))

(defn pull-times
  ([s]
     (let [[match times-etc] (match-globs #" at (.*)" s)]
       (if times-etc
         (pull-times times-etc #{})
         {:times nil
          :times-rest s})))
  ([s times]
     (let [time-re #"((?:, )?(?:and )?(\d{1,2}):(\d{2})).*"
           [match root hour-str minute-str] (match-globs time-re s)]
       (if-not (and hour-str minute-str)
         {:times times
          :times-rest (.substring s (count root))}
         (recur (.substring s (count root))
                (conj times [(string->int hour-str) (string->int minute-str)]))))))

(defn read-floating-pattern
        [s]
        (let [{:keys [days days-rest]} (pull-days s)
              {:keys [times times-rest]} (pull-times days-rest)
              tz-re #"(?: (.+))?"
              [match tz] (match-globs tz-re times-rest)]
          (floating-pattern days times tz)))

                                                                                    
                                                                             
;;;;;;;;;;;; This file autogenerated from cljx/src/schedule.cljx

==> schedule-0.1.4.jar: schedule.clj <==
(ns schedule
        (:require [clojure.string :as str])
        (:import clojure.lang.IDeref
                 clojure.lang.Seqable
                 clojure.lang.Cons
                 clojure.lang.ILookup
                 java.util.Date
                 java.util.Calendar
                 java.util.TimeZone
                 java.io.Writer)
                                           
                                            )

(defprotocol Pattern
  (anchor  [pattern t]    "Anchors the scheduling pattern at time `t`, yielding a schedule starting after `t`"))

(defprotocol WeeklyTriggered
  (at-hour [pattern hour] "Return a new pattern that only generates instants at hour `hour`")
  (add-hour [pattern hour] "Returns a new pattern that generates instants at hour `hour`")
  (remove-hour [pattern hour] "Returns a new pattern that will not generate instants at hour `hour`")
  (at-time [pattern hour minute] "Return a new pattern that only generates instants at the time indicated by `hour` and `minute`")
  (add-time [pattern hour minute] "Returns a new pattern that generates instants at the time indicated by `hour` and `minute")
  (remove-time [pattern hour minute] "Returns a new pattern that will not generate instants at the time indicated by `hour` and `minute`"))

(defprotocol TimeZoneable
  (in-tz [timezonable tz] "Shifts timezoneable, returning a new object projected in timezoe `tz`."))

(defprotocol Instant
  (as-ts [instant] "Return a plaform native timestamp type corresponding to this instant.")
  (as-millis [instant] "Return a long representing the number of millis since the epoch corresponding to this instant."))

(defn current-time-millis
  []
        (System/currentTimeMillis)
                       
  )

      (extend-protocol Instant
        Date
        (as-ts [date] date)
        (as-millis [date] (.getTime date))
        Long
        (as-ts [long] (Date. long))
        (as-millis [long] long)
        Calendar
        (as-ts [cal] (.getTime cal))
        (as-millis [cal] (.getTimeInMillis cal)))

                               
                
                            
                                           
               
                                     
                               

(declare ^:private update-schedule-pattern)
(declare schedule-lazy-seq)

                      
                 
                 
                 
                 
                 
                 
                 
                   

(defn roll-to
  "Rolls ms forward, finding the next time ms which occurs on day at
  hour and minute in tz"
  [ms day hour minute tz]
        (let [weekday-map {:monday Calendar/MONDAY
                           :tuesday Calendar/TUESDAY
                           :wednesday Calendar/WEDNESDAY
                           :thursday Calendar/THURSDAY
                           :friday Calendar/FRIDAY
                           :saturday Calendar/SATURDAY
                           :sunday Calendar/SUNDAY}
              cal (Calendar/getInstance)]
          (.setTimeInMillis cal ms)
          (.set cal Calendar/DAY_OF_WEEK (weekday-map day))
          (.set cal Calendar/HOUR_OF_DAY hour)
          (.set cal Calendar/MINUTE minute)
          (.set cal Calendar/SECOND 0)
          (.set cal Calendar/MILLISECOND 0)
          (.setTimeZone cal (TimeZone/getTimeZone (or tz "UTC")))
          (if (<= ms (.getTimeInMillis cal))
            (.getTimeInMillis cal)
            (do (.add cal Calendar/DAY_OF_YEAR 7)
                (.getTimeInMillis cal))))
                                     
                                      
                                        
                                       
                                     
                                       
                                      
                              
                                               
                                                              
                                                           
                                                          
                                                                   
                                                       
                                              
                                         
                                           
                                                             
                                                               
                                           
                                 
                                                              
                                        )

(defn- calculate-next-schedule-instant
  [{{:keys [tz time-matches days] :as pattern} :pattern
    start :start
    :as schedule}]
  (let [days (or days
                 [:monday :tuesday :wednesday :thursday :friday :saturday :sunday])
        candidates (for [day days [hour min] time-matches]
                     (roll-to start day hour min tz))]
    (apply min candidates)))

(defn print-to-writer
  [val writer opts]
                                     
        (binding [*out* writer]
          (pr val)))

(defn print-weekly-schedule-to-abstract-writer
  [{:keys [pattern start] :as schedule} writer write-fn opts]
  (write-fn writer "#schedule/weekly-schedule ")
  (print-to-writer {:pattern pattern
                    :start (as-ts start)} writer opts))

(defn- lookup-schedule-key
  ([schedule key] (lookup-schedule-key schedule key nil))
  ([schedule key not-found]
     (let [lookup (condp get key
                    #{:pattern 'pattern} (.-pattern schedule)
                    #{:start 'start} (.-start schedule)
                    ::not-found)]
       (if (= lookup ::not-found)
         not-found
         lookup))))

(declare schedules-equiv?)

(deftype WeeklySchedule [pattern start]
  Pattern
  (anchor  [this t]    (WeeklySchedule. pattern (as-millis t)))
  WeeklyTriggered
  (at-hour [this hour] (update-schedule-pattern at-hour hour))
  (add-hour [this hour] (update-schedule-pattern add-hour hour))
  (remove-hour [this hour] (update-schedule-pattern remove-hour hour))
  (at-time [this hour minute] (update-schedule-pattern at-time hour minute))
  (add-time [this hour minute] (update-schedule-pattern add-time hour minute))
  (remove-time [this hour minute] (update-schedule-pattern remove-time hour minute))
  TimeZoneable
  (in-tz   [this tz]   (update-schedule-pattern in-tz tz))
        Object
        (equals [this schedule] (schedules-equiv? this schedule))
               
                                                                  
        ILookup
        (valAt [this key] (lookup-schedule-key this key))
        (valAt [this key not-found] (lookup-schedule-key this key not-found))
                
                                                            
                                                                                
        Seqable
        (seq [this] (schedule-lazy-seq this))
                 
                                               
                         
                                           
                                                                                  )

(defn schedules-equiv?
  [schedule1 schedule2]
  (and (instance? WeeklySchedule schedule1)
       (instance? WeeklySchedule schedule2)
       (= (:pattern schedule1) (:pattern schedule2))
       (= (:start schedule1) (:start schedule2))))

      (defmethod print-method WeeklySchedule
        [^WeeklySchedule schedule ^Writer w]
        (print-weekly-schedule-to-abstract-writer schedule w #(.write %1 %2) nil))

(defn schedule-lazy-seq
  [{:keys [pattern] :as schedule}]
  (let [next-instant-ms (calculate-next-schedule-instant schedule)]
    (cons (as-ts next-instant-ms)
          (lazy-seq (WeeklySchedule. pattern (inc next-instant-ms))))))

(defn- update-schedule-pattern
  [{:keys [pattern start]} f & args]
  (WeeklySchedule. (apply f pattern args) start))

(defn- write-n
  [coll writer write-fn serial-fn empty-fn]
  (case (count coll)
    0 (write-fn writer (empty-fn))
    1 (write-fn writer (serial-fn (first coll)))
    2 (do (write-fn writer (serial-fn (first coll)))
          (write-fn writer ", and ")
          (write-fn writer (serial-fn (second coll))))
    (do (doseq [item (butlast coll)]
          (write-fn writer (serial-fn item))
          (write-fn writer ", "))
        (write-fn writer "and ")
        (write-fn writer (serial-fn (last coll))))))

(defn- print-weekly-pattern-to-abstract-writer
  [{:keys [days time-matches tz] :as pattern} writer write-fn opts]
  (write-fn writer "#schedule/weekly-pattern \"")
  (write-n days writer write-fn #(str/capitalize (name %)) (constantly "Every day"))
  (when time-matches
    (write-fn writer " at "))
  (write-n (sort time-matches) writer write-fn #(let [[hour minute] %]
                                                  (str hour ":" (when (< minute 10) "0") minute))
           (constantly ""))
  (when tz
    (write-fn writer " ")
    (write-fn writer tz))
  (write-fn writer "\""))

(defn- lookup-pattern-key
  ([pattern key] (lookup-pattern-key pattern key nil))
  ([pattern key not-found]
     (let [lookup (condp get key
                    #{:days 'days} (.-days pattern)
                    #{:time-matches 'time-matches} (.-time-matches pattern)
                    #{:tz 'tz} (.-tz pattern)
                    ::not-found)]
       (if (= lookup ::not-found)
         not-found
         lookup))))

(declare patterns-equiv?)

(deftype WeeklyPattern [days time-matches tz]
  Pattern
  (anchor  [pattern t]    (WeeklySchedule. pattern (as-millis t)))
  WeeklyTriggered
  (at-hour [pattern new-hour] (WeeklyPattern. days #{[new-hour 0]} tz))
  (add-hour [pattern new-hour] (WeeklyPattern. days (conj time-matches [new-hour 0]) tz))
  (remove-hour [pattern del-hour] (WeeklyPattern. days (disj time-matches [del-hour 0]) tz))
  (at-time [pattern new-hour new-minute] (WeeklyPattern. days #{[new-hour new-minute]} tz))
  (add-time [pattern new-hour new-minute] (WeeklyPattern. days (conj time-matches [new-hour new-minute]) tz))
  (remove-time [pattern del-hour del-minute] (WeeklyPattern. days (disj time-matches [del-hour del-minute]) tz))
  TimeZoneable
  (in-tz   [pattern new-tz]   (WeeklyPattern. days time-matches new-tz))
        Object
        (equals [this pattern] (patterns-equiv? this pattern))
               
                                                               
        ILookup
        (valAt [this key] (lookup-pattern-key this key))
        (valAt [this key not-found] (lookup-pattern-key this key not-found))
                
                                                           
                                                                               
        IDeref
        (deref [pattern] (anchor pattern (current-time-millis)))
               
                                                                  
                         
                                          
                                                                                )

(defn patterns-equiv?
  [pattern1 pattern2]
  (and (instance? WeeklyPattern pattern1)
       (instance? WeeklyPattern pattern2)
       (= (:days pattern1) (:days pattern2))
       (= (:time-matches pattern1) (:time-matches pattern2))
       (= (:tz pattern1) (:tz pattern2))))

      (defmethod print-method WeeklyPattern
        [^WeeklyPattern pattern ^Writer w]
        (print-weekly-pattern-to-abstract-writer pattern w #(.write %1 %2) nil))

(defn read-schedule
  [m]
  (let [pattern (:pattern m)
        start (:start m)]
    (WeeklySchedule. pattern (as-millis start))))

(defn- string->int
  [int]
  (when int
          (Integer/parseInt int)
                               ))

(defn- floating-pattern
  [days time-matches tz]
  (WeeklyPattern. days time-matches tz))

(defn- match-globs
  [re s]
        (let [matcher (re-matcher re s)]
          (re-find matcher))
                          )

(defn pull-days
  ([s]
     (pull-days s nil))
  ([s days]
     (let [day-re #"((?:, )?(?:and )?((?:Every |Mon|Tues|Wednes|Thurs|Fri|Satur|Sun)day)).*"
           [match root first-day] (match-globs day-re s)]
       (if (or (= first-day "Every day")
               (nil? first-day))
         {:days days
          :days-rest (.substring s (count root))}
         (recur (.substring s (count root))
                (conj days (keyword (.toLowerCase first-day))))))))

(defn pull-times
  ([s]
     (let [[match times-etc] (match-globs #" at (.*)" s)]
       (if times-etc
         (pull-times times-etc #{})
         {:times nil
          :times-rest s})))
  ([s times]
     (let [time-re #"((?:, )?(?:and )?(\d{1,2}):(\d{2})).*"
           [match root hour-str minute-str] (match-globs time-re s)]
       (if-not (and hour-str minute-str)
         {:times times
          :times-rest (.substring s (count root))}
         (recur (.substring s (count root))
                (conj times [(string->int hour-str) (string->int minute-str)]))))))

(defn read-floating-pattern
        [s]
        (let [{:keys [days days-rest]} (pull-days s)
              {:keys [times times-rest]} (pull-times days-rest)
              tz-re #"(?: (.+))?"
              [match tz] (match-globs tz-re times-rest)]
          (floating-pattern days times tz)))

                                                                                    
                                                                             
;;;;;;;;;;;; This file autogenerated from cljx/src/schedule.cljx
